--
-- wetDamaging
-- Specialization for wetDamaging
--
-- @author:		Michi77
-- @date:		10/07/16
-- @version:	v1.0
-- @history:	v1.0 - initial version
--
-- free for non-commercial usage

wetDamaging = {};

function wetDamaging.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Washable, specializations);
end;

function wetDamaging:load(xmlFile)
	self.addExceptionFruit = SpecializationUtil.callSpecializationsFunction("addExceptionFruit");
	self.checkFruit = wetDamaging.checkFruit;
	--self.checkFruit = SpecializationUtil.callSpecializationsFunction("checkFruit");
	
	self.addRainType = SpecializationUtil.callSpecializationsFunction("addRainType");
	self.removeRainType = SpecializationUtil.callSpecializationsFunction("removeRainType");
	self.checkRain = wetDamaging.checkRain;
	--self.checkRain = SpecializationUtil.callSpecializationsFunction("checkRain");
	
	
	self.rainTypes = {};
	self:addRainType("rain");
	self:addRainType("hail");
	
	self.currentDamage = 0;
	self.currentNightDamage = 0;
	self.dryTime = 240; -- 4 hours
	self.damageTime = 30; -- 30 minutes
	
	self.exceptionFruits = {};
	self:addExceptionFruit(FruitUtil.FRUITTYPE_SUGARBEET);
	self:addExceptionFruit(FruitUtil.FRUITTYPE_POTATO);
	self:addExceptionFruit(FruitUtil.FRUITTYPE_MAIZE);
	self:addExceptionFruit(FruitUtil.FRUITTYPE_CHAFF);
	self:addExceptionFruit(FruitUtil.FRUITTYPE_GRASS);
	
	self.lastRainTime = 0;
	self.damageBeforeRain = 0;
	self.savedLastRainScale = 0; 
	self.maxWetDamage = 0.25;
	self.maxNightDamage = 0.05;
	self.oldTime = 0;
	
	self.allowThreshingDuringRain = true;
	self.origThreshingScale = self.threshingScale;
end;

function wetDamaging:mouseEvent(posX, posY, isDown, isUp, button)
end;

function wetDamaging:keyEvent(unicode, sym, modifier, isDown)
end;

function wetDamaging:update(dt)
end;

function wetDamaging:updateTick(dt)
	if self.fillLevel > 0 and self.currentFillType ~= FruitUtil.FRUITTYPE_UNKNOWN and not self:checkFruit(self.currentFillType) then
		local g = g_currentMission.environment;
		if g.currentRain ~= nil and self:checkRain(g.currentRain.rainTypeId) and g.lastRainScale > 0.01 and self.lastRainType == nil then
			self.lastRainType = g.currentRain.rainTypeId;
		elseif g.timeSinceLastRain > self.dryTime and self.lastRainType ~= nil then
			self.lastRainType = nil;
			self.damageBeforeRain = 0;
			
		elseif self.lastRainType ~= nil then
			self.lastRainTime = 0;
			self.savedLastRainScale = 0;
			self.damageBeforeRain = self.currentDamage;
		end;
		
		if self.lastRainType ~= nil then
			if g.lastRainScale > 0.01 then
				local dtMinutes = dt / 60000 * g.timeScale;
				
				self.lastRainTime = self.lastRainTime + dtMinutes;
				
				
				if g.lastRainScale > self.savedLastRainScale then
					self.savedLastRainScale = g.lastRainScale;
				end;
				
				local damageTime = (self.lastRainTime / 2) * self.savedLastRainScale;
				self.currentDamage = math.min((damageTime / self.damageTime) + self.damageBeforeRain, self.maxWetDamage);
			end;
			
			if g.timeSinceLastRain > 0 then
				local dryTime = self.dryTime - g.timeSinceLastRain;
				self.currentDamage = math.max((dryTime / self.dryTime) / 4, 0);
				
			end;
		end;
		
		if ((g_currentMission.environment.dayTime < 86399904 and g_currentMission.environment.dayTime > 75602024) or g_currentMission.environment.dayTime < 21605768) then
			self.currentNightDamage = self.maxNightDamage;
		end;
		
		self.threshingScale = self.origThreshingScale - (self.currentDamage + self.currentNightDamage);
	end;
end;

function wetDamaging:draw()
end;

function wetDamaging:addExceptionFruit(fruit)
	self.exceptionFruits[fruit] = true;
end;

function wetDamaging:checkFruit(fruit, vehicle)
	if self.exceptionFruits[fruit] ~= nil and self.exceptionFruits[fruit] then
		return true;
	end;
	return false;
end;

function wetDamaging:addRainType(rain)
	self.rainTypes[rain] = true;
end;

function wetDamaging:removeRainType(rain)
	self.rainTypes[rain] = nil;
end;

function wetDamaging:checkRain(rainType)
	if self.rainTypes[rainType] ~= nil then
		return true;
	end;
	return false;
end;

function wetDamaging:delete()
end;
